@@ -150,28 +150,37 @@ public class JdkClassReader {
         return classModel;
     }
 
-
+    /**
+     * Returns the access flags of this class as stored in the class file.
+     *
+     * @return The access flags of this class as stored in the class file.
+     */
     public int getAccess() {
         return classModel.flags().flagsMask();
     }
 
+    /**
+     * Returns the internal name of this class.
+     *
+     * @return The internal name of this class.
+     */
     public String getClassName() {
         return classModel.thisClass().asInternalName();
     }
 
     /**
-     * Returns the super class name of this class or {@code null} for {@link Object}.
+     * Returns the internal super class name of this class or {@code null} for {@link Object}.
      *
-     * @return The super class name of this class or {@code null} for {@link Object}.
+     * @return The internal super class name of this class or {@code null} for {@link Object}.
      */
     public String getSuperName() {
         return classModel.superclass().map(ClassEntry::asInternalName).orElse(null);
     }
 
     /**
-     * Returns the interface names of this class.
+     * Returns the internal interface names of this class. Maybe {@code null}.
      *
-     * @return The interface names of this class.
+     * @return The internal interface names of this class.
      */
     public String[] getInterfaces() {
         return classModel.interfaces().stream().map(ClassEntry::asInternalName).toArray(String[]::new);
@@ -738,7 +747,7 @@ public class JdkClassReader {
 
     private static TypeReference toTypeReference(TypeAnnotation.TargetInfo targetInfo) {
         return switch (targetInfo) {
-            case TypeAnnotation.SupertypeTarget value -> TypeReference.newSuperTypeReference(value.supertypeIndex());
+            case TypeAnnotation.SupertypeTarget value -> TypeReference.newSuperTypeReference(value.supertypeIndex() == 0xFFFF ? -1 : value.supertypeIndex());
             case TypeAnnotation.TypeParameterTarget value -> TypeReference.newTypeParameterReference(value.targetType().targetTypeValue(), value.typeParameterIndex());
             case TypeAnnotation.TypeParameterBoundTarget value -> TypeReference.newTypeParameterBoundReference(value.targetType().targetTypeValue(), value.typeParameterIndex(), value.boundIndex());
             case TypeAnnotation.ThrowsTarget value -> TypeReference.newExceptionReference(value.throwsTargetIndex());
